function [nErr] = LMwritem(szInputFile,A,b,c,csense,lb,ub,fformat,osense,vtype)
% LMWRITEM: Export a model in matrix form in MPS or LINDO format.
%
% Usage:  [nErr] = LMwritem(szInputFile,A,b,c,csense,lb,ub,fformat,osense,vtype)
%
% Copyright (c) 2001-2007
%
% LINDO Systems, Inc.            312.988.7422
% 1415 North Dayton St.          info@lindo.com
% Chicago, IL 60622              http://www.lindo.com    
%

% Last update Jan 09, 2007 (MKA)
%
global MY_LICENSE_FILE
lindo;

if nargin < 10,
   vtype =[];
   if nargin < 9,
      osense = LS_MIN;
      if nargin < 8,
         fformat = 0;         
         if nargin < 7,
            ub = [];
            if nargin < 6,
                lb =[];
                if nargin < 5,
                   fprintf('Incomplete model data\n');
                   return;
                end;
            end;
         end;
      end;
   end;
end;

fixed_obj = 0;   % assumed fixed objective = 0
verbose = 0;

% if constraint senses are not given, all assumed to be 'E'
if (isempty(csense)) 
   for i=1:m, csense=[csense 'E']; end;
end;

% Read license key from a license file
[MY_LICENSE_KEY,nErr] = mxlindo('LSloadLicenseString',MY_LICENSE_FILE);

% Create a LINDO environment
[iEnv,nErr]=mxlindo('LScreateEnv',MY_LICENSE_KEY);
if nErr ~= LSERR_NO_ERROR, LMcheckError(iEnv,nErr) ; return; end;

% Declare and create a model 
[imodel,nErr]=mxlindo('LScreateModel',iEnv);
if nErr ~= LSERR_NO_ERROR, LMcheckError(iEnv,nErr) ; return; end;

% Load the data 
if (~issparse(A)),  A = sparse(A); end; 
if osense == -1, c = -c; end;
[nErr]=mxlindo('LSXloadLPData',imodel,osense,fixed_obj,c,b,csense,A,lb,ub);
if nErr ~= LSERR_NO_ERROR, LMcheckError(iEnv,nErr) ; return; end;

if (~isempty(vtype))
[nErr]=mxlindo('LSloadMIPData',imodel,vtype);
if nErr ~= LSERR_NO_ERROR, LMcheckError(iEnv,nErr) ; return; end;
end;

if fformat == 0,
   [nErr]=mxlindo('LSwriteMPSFile',imodel,szInputFile,LS_UNFORMATTED_MPS);
else 
   [nErr]=mxlindo('LSwriteLINDOFile',imodel,szInputFile);
end;
if nErr ~= LSERR_NO_ERROR, LMcheckError(iEnv,nErr) ; return; end;
 
% Un-hook
[nErr]=mxlindo('LSdeleteModel',imodel);
if nErr ~= LSERR_NO_ERROR, LMcheckError(iEnv,nErr) ; return; end;

[nErr]=mxlindo('LSdeleteEnv',iEnv);
if nErr ~= LSERR_NO_ERROR, LMcheckError(iEnv,nErr) ; return; end;